/*!
 * Audio/Video tracking for Snowplow v4.5.0 (http://bit.ly/sp-js)
 * Copyright 2022 Snowplow Analytics Ltd
 * Licensed under BSD-3-Clause
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.snowplowMediaTracking = {}));
})(this, (function (exports) { 'use strict';

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
  };

  function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
  }

  typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
  };

  /*!
   * Core functionality for Snowplow JavaScript trackers v4.5.0 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  function payloadBuilder() {
      var dict = {}, allJson = [], jsonForProcessing = [], contextEntitiesForProcessing = [];
      var processor;
      var add = function (key, value) {
          if (value != null && value !== '') {
              // null also checks undefined
              dict[key] = value;
          }
      };
      var addDict = function (dict) {
          for (var key in dict) {
              if (Object.prototype.hasOwnProperty.call(dict, key)) {
                  add(key, dict[key]);
              }
          }
      };
      var addJson = function (keyIfEncoded, keyIfNotEncoded, json) {
          if (json && isNonEmptyJson(json)) {
              var jsonWithKeys = { keyIfEncoded: keyIfEncoded, keyIfNotEncoded: keyIfNotEncoded, json: json };
              jsonForProcessing.push(jsonWithKeys);
              allJson.push(jsonWithKeys);
          }
      };
      var addContextEntity = function (entity) {
          contextEntitiesForProcessing.push(entity);
      };
      return {
          add: add,
          addDict: addDict,
          addJson: addJson,
          addContextEntity: addContextEntity,
          getPayload: function () { return dict; },
          getJson: function () { return allJson; },
          withJsonProcessor: function (jsonProcessor) {
              processor = jsonProcessor;
          },
          build: function () {
              processor === null || processor === void 0 ? void 0 : processor(this, jsonForProcessing, contextEntitiesForProcessing);
              return dict;
          },
      };
  }
  /**
   * Is property a non-empty JSON?
   * @param property - Checks if object is non-empty json
   */
  function isNonEmptyJson(property) {
      if (!isJson(property)) {
          return false;
      }
      for (var key in property) {
          if (Object.prototype.hasOwnProperty.call(property, key)) {
              return true;
          }
      }
      return false;
  }
  /**
   * Is property a JSON?
   * @param property - Checks if object is json
   */
  function isJson(property) {
      return (typeof property !== 'undefined' &&
          property !== null &&
          (property.constructor === {}.constructor || property.constructor === [].constructor));
  }

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  var label = 'Snowplow: ';
  var LOG_LEVEL;
  (function (LOG_LEVEL) {
      LOG_LEVEL[LOG_LEVEL["none"] = 0] = "none";
      LOG_LEVEL[LOG_LEVEL["error"] = 1] = "error";
      LOG_LEVEL[LOG_LEVEL["warn"] = 2] = "warn";
      LOG_LEVEL[LOG_LEVEL["debug"] = 3] = "debug";
      LOG_LEVEL[LOG_LEVEL["info"] = 4] = "info";
  })(LOG_LEVEL || (LOG_LEVEL = {}));
  var LOG$1 = logger();
  function logger(logLevel) {
      if (logLevel === void 0) { logLevel = LOG_LEVEL.warn; }
      function setLogLevel(level) {
          if (LOG_LEVEL[level]) {
              logLevel = level;
          }
          else {
              logLevel = LOG_LEVEL.warn;
          }
      }
      /**
       * Log errors, with or without error object
       */
      function error(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.error && typeof console !== 'undefined') {
              var logMsg = label + message + '\n';
              if (error) {
                  console.error.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.error.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log warnings, with or without error object
       */
      function warn(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.warn && typeof console !== 'undefined') {
              var logMsg = label + message;
              if (error) {
                  console.warn.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.warn.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log debug messages
       */
      function debug(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.debug && typeof console !== 'undefined') {
              console.debug.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      /**
       * Log info messages
       */
      function info(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.info && typeof console !== 'undefined') {
              console.info.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      return { setLogLevel: setLogLevel, warn: warn, error: error, debug: debug, info: info };
  }
  /**
   * Find dynamic context generating functions and return their results to be merged into the static contexts
   * Combine an array of unchanging contexts with the result of a context-creating function
   *
   * @param dynamicOrStaticContexts - Array of custom context Objects or custom context generating functions
   * @param Parameters - to pass to dynamic context callbacks
   * @returns An array of Self Describing JSON context
   */
  function resolveDynamicContext(dynamicOrStaticContexts) {
      var _a;
      var extraParams = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          extraParams[_i - 1] = arguments[_i];
      }
      return ((_a = dynamicOrStaticContexts === null || dynamicOrStaticContexts === void 0 ? void 0 : dynamicOrStaticContexts.map(function (context) {
          if (typeof context === 'function') {
              try {
                  return context.apply(void 0, extraParams);
              }
              catch (e) {
                  //TODO: provide warning
                  return undefined;
              }
          }
          else {
              return context;
          }
      }).filter(Boolean)) !== null && _a !== void 0 ? _a : []);
  }
  /**
   * Build a self-describing event
   * A custom event type, allowing for an event to be tracked using your own custom schema
   * and a data object which conforms to the supplied schema
   *
   * @param event - Contains the properties and schema location for the event
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildSelfDescribingEvent(event) {
      var _a = event.event, schema = _a.schema, data = _a.data, pb = payloadBuilder();
      var ueJson = {
          schema: 'iglu:com.snowplowanalytics.snowplow/unstruct_event/jsonschema/1-0-0',
          data: { schema: schema, data: data },
      };
      pb.add('e', 'ue');
      pb.addJson('ue_px', 'ue_pr', ueJson);
      return pb;
  }

  var SEARCH_ERROR;
  (function (SEARCH_ERROR) {
      SEARCH_ERROR["NOT_FOUND"] = "Media element not found";
      SEARCH_ERROR["MULTIPLE_ELEMENTS"] = "More than one media element in the provided node";
      SEARCH_ERROR["PLYR_CURRENTSRC"] = "Plyr currentSrc not updated";
  })(SEARCH_ERROR || (SEARCH_ERROR = {}));
  var READY_STATE = {
      0: 'HAVE_NOTHING',
      1: 'HAVE_METADATA',
      2: 'HAVE_CURRENT_DATA',
      3: 'HAVE_FUTURE_DATA',
      4: 'HAVE_ENOUGH_DATA',
  };
  var NETWORK_STATE = {
      0: 'NETWORK_EMPTY',
      1: 'NETWORK_IDLE',
      2: 'NETWORK_LOADING',
      3: 'NETWORK_NO_SOURCE',
  };

  function isFullScreen(el) {
      return document.fullscreenElement === el || false;
  }
  function isHtmlMediaElement(element) {
      return element instanceof HTMLMediaElement;
  }
  function isHtmlAudioElement(element) {
      return element instanceof HTMLAudioElement;
  }
  function isHtmlVideoElement(element) {
      return element instanceof HTMLVideoElement;
  }
  function getUriFileExtension(uri) {
      // greedily match until the final '.' is found with trailing characters, capture those as extension
      // only capture until finding URI metacharacters
      var pattern = /.+\.([^\.?&#]+)/;
      var uriPath = uri;
      try {
          uriPath = new URL(uri).pathname;
      }
      catch (e) { }
      // try to match against the pathname only first, if unsuccessful try against the full URI
      var match = pattern.exec(uriPath) || pattern.exec(uri);
      return match && match[1];
  }
  function getDuration(el) {
      var duration = el.duration;
      // A NaN value is returned if duration is not available, or Infinity if the media resource is streaming.
      if (isNaN(duration) || duration === Infinity) {
          return null;
      }
      return duration;
  }
  // Checks if a url is a data_url, so we don't send a (potentially large) payload
  function dataUrlHandler(url) {
      if (url.indexOf('data:') !== -1) {
          return 'DATA_URL';
      }
      return url;
  }
  function timeRangesToObjectArray(t) {
      var out = [];
      for (var i = 0; i < t.length; i++) {
          out.push({ start: t.start(i), end: t.end(i) });
      }
      return out;
  }
  function textTrackListToJson(textTrackList) {
      return Object.keys(textTrackList).map(function (_, i) {
          return {
              label: textTrackList[i].label,
              language: textTrackList[i].language,
              kind: textTrackList[i].kind,
              mode: textTrackList[i].mode,
          };
      });
  }
  function setConfigDefaults(config) {
      var defaults = {
          boundaries: [10, 25, 50, 75],
      };
      return __assign(__assign({}, defaults), config);
  }
  function parseVolume(volume) {
      return parseInt((volume * 100).toString());
  }

  /**
   * Waits for an HTML media element and invokes a callback with the element once found.
   *
   * - First, it tries to find the media element by id
   *   - If found and valid (`isHtmlMediaElement()`), the callback is called with the updated config.
   *   - If not found, a warning is logged and a `MutationObserver` is set up.
   *
   * - The `MutationObserver` watches for new elements added to the DOM. When the target media element is detected,
   *   the callback is invoked, and the observer disconnects.
   *
   * Useful for cases where the media element might not be in the DOM immediately.
   */
  function waitForElement(config, callback) {
      var _a = findMediaElement(config.video), el = _a.el, err = _a.err;
      if (err) {
          LOG$1.info("".concat(err, ". Waiting for element to be added to the DOM."));
      }
      if (isHtmlMediaElement(el)) {
          callback(__assign(__assign({}, config), { video: el }));
      }
      else {
          var observer_1 = new MutationObserver(function (mutations) {
              mutations.forEach(function (mut) {
                  if (!mut.addedNodes) {
                      return;
                  }
                  for (var _i = 0, _a = Object.values(mut.addedNodes); _i < _a.length; _i++) {
                      var node = _a[_i];
                      if (isHtmlMediaElement(node) && node.id === config.video) {
                          callback(__assign(__assign({}, config), { video: node }));
                          observer_1.disconnect();
                      }
                  }
              });
          });
          observer_1.observe(document.body, { childList: true, subtree: true });
      }
  }
  function findMediaElement(id) {
      var el = document.getElementById(id);
      if (!el)
          return { err: SEARCH_ERROR.NOT_FOUND };
      if (isHtmlAudioElement(el))
          return { el: el };
      if (isHtmlVideoElement(el)) {
          // Plyr loads in an initial blank video with currentSrc as https://cdn.plyr.io/static/blank.mp4
          // so we need to check until currentSrc updates.
          if (el.currentSrc === 'https://cdn.plyr.io/static/blank.mp4' && el.readyState === 0) {
              return { err: SEARCH_ERROR.PLYR_CURRENTSRC };
          }
          return { el: el };
      }
      return findMediaElementChild(el);
  }
  function findMediaElementChild(el) {
      for (var _i = 0, _a = ['VIDEO', 'AUDIO']; _i < _a.length; _i++) {
          var tag = _a[_i];
          var descendentTags = el.getElementsByTagName(tag);
          if (descendentTags.length === 1) {
              var el_1 = descendentTags[0];
              if (isHtmlAudioElement(el_1) || isHtmlVideoElement(el_1)) {
                  return { el: el_1 };
              }
          }
          else if (descendentTags.length === 2 && tag === 'VIDEO' && isHtmlVideoElement(descendentTags[0])) {
              // Special JWPlayer case where two video elements are used for cover-video effect.
              // In that case, we select the first video element.
              return { el: descendentTags[0] };
          }
          else if (descendentTags.length > 1) {
              return { err: SEARCH_ERROR.MULTIPLE_ELEMENTS };
          }
      }
      return { err: SEARCH_ERROR.NOT_FOUND };
  }

  function isStringConfig(config) {
      return typeof config.video === 'string';
  }
  function isElementConfig(config) {
      return config.video instanceof HTMLMediaElement;
  }

  /*!
   * Core functionality for Snowplow Browser trackers v4.5.0 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */
  /**
   * Dispatch function to all specified trackers from the supplied collection
   *
   * @param trackers - An optional list of trackers to send the event to, or will send to all trackers
   * @param trackerCollection - The collection which the trackers will be selected from
   * @param fn - The function which will run against each tracker
   */
  function dispatchToTrackersInCollection(trackers, trackerCollection, fn) {
      try {
          getTrackersFromCollection(trackers !== null && trackers !== void 0 ? trackers : Object.keys(trackerCollection), trackerCollection).forEach(fn);
      }
      catch (ex) {
          LOG$1.error('Function failed', ex);
      }
  }
  function getTrackersFromCollection(trackerIds, trackerCollection) {
      var trackers = [];
      for (var _i = 0, trackerIds_1 = trackerIds; _i < trackerIds_1.length; _i++) {
          var id = trackerIds_1[_i];
          if (trackerCollection.hasOwnProperty(id)) {
              trackers.push(trackerCollection[id]);
          }
          else {
              LOG$1.warn(id + ' not configured');
          }
      }
      return trackers;
  }

  /*!
   * Snowplow media tracking v4.5.0 (https://github.com/snowplow/snowplow-javascript-tracker)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */

  /** Type of media player event */
  var MediaEventType$1;
  (function (MediaEventType) {
      // Controlling the playback
      /** Media player event fired when the media tracking is successfully attached to the player and can track events. */
      MediaEventType["Ready"] = "ready";
      /** Media player event sent when the player changes state to playing from previously being paused. */
      MediaEventType["Play"] = "play";
      /** Media player event sent when the user pauses the playback. */
      MediaEventType["Pause"] = "pause";
      /** Media player event sent when playback stops when end of the media is reached or because no further data is available. */
      MediaEventType["End"] = "end";
      /** Media player event sent when a seek operation begins. */
      MediaEventType["SeekStart"] = "seek_start";
      /** Media player event sent when a seek operation completes. */
      MediaEventType["SeekEnd"] = "seek_end";
      // Changes in playback settings
      /** Media player event sent when the playback rate has changed. */
      MediaEventType["PlaybackRateChange"] = "playback_rate_change";
      /** Media player event sent when the volume has changed. */
      MediaEventType["VolumeChange"] = "volume_change";
      /** Media player event fired immediately after the browser switches into or out of full-screen mode. */
      MediaEventType["FullscreenChange"] = "fullscreen_change";
      /** Media player event fired immediately after the browser switches into or out of picture-in-picture mode. */
      MediaEventType["PictureInPictureChange"] = "picture_in_picture_change";
      // Tracking playback progress
      /** Media player event fired periodically during main content playback, regardless of other API events that have been sent. */
      MediaEventType["Ping"] = "ping";
      /** Media player event fired when a percentage boundary set in options.boundaries is reached */
      MediaEventType["PercentProgress"] = "percent_progress";
      // Ad events
      /** Media player event that signals the start of an ad break. */
      MediaEventType["AdBreakStart"] = "ad_break_start";
      /** Media player event that signals the end of an ad break. */
      MediaEventType["AdBreakEnd"] = "ad_break_end";
      /** Media player event that signals the start of an ad. */
      MediaEventType["AdStart"] = "ad_start";
      /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdFirstQuartile"] = "ad_first_quartile";
      /** Media player event fired when a midpoint of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdMidpoint"] = "ad_midpoint";
      /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdThirdQuartile"] = "ad_third_quartile";
      /** Media player event that signals the ad creative was played to the end at normal speed. */
      MediaEventType["AdComplete"] = "ad_complete";
      /** Media player event fired when the user activated a skip control to skip the ad creative. */
      MediaEventType["AdSkip"] = "ad_skip";
      /** Media player event fired when the user clicked on the ad. */
      MediaEventType["AdClick"] = "ad_click";
      /** Media player event fired when the user clicked the pause control and stopped the ad creative. */
      MediaEventType["AdPause"] = "ad_pause";
      /** Media player event fired when the user resumed playing the ad creative after it had been stopped or paused. */
      MediaEventType["AdResume"] = "ad_resume";
      // Data quality events
      /** Media player event fired when the player goes into the buffering state and begins to buffer content. */
      MediaEventType["BufferStart"] = "buffer_start";
      /** Media player event fired when the the player finishes buffering content and resumes playback. */
      MediaEventType["BufferEnd"] = "buffer_end";
      /** Media player event tracked when the video playback quality changes automatically. */
      MediaEventType["QualityChange"] = "quality_change";
      /** Media player event tracked when the resource could not be loaded due to an error.  */
      MediaEventType["Error"] = "error";
  })(MediaEventType$1 || (MediaEventType$1 = {}));
  /** Type of media content. */
  var MediaType$1;
  (function (MediaType) {
      /** Audio content. */
      MediaType["Audio"] = "audio";
      /** Video content. */
      MediaType["Video"] = "video";
  })(MediaType$1 || (MediaType$1 = {}));
  /** Type of ads within the break */
  var MediaAdBreakType$1;
  (function (MediaAdBreakType) {
      /** Take full control of the video for a period of time. */
      MediaAdBreakType["Linear"] = "linear";
      /** Run concurrently to the video. */
      MediaAdBreakType["NonLinear"] = "nonlinear";
      /** Accompany the video but placed outside the player. */
      MediaAdBreakType["Companion"] = "companion";
  })(MediaAdBreakType$1 || (MediaAdBreakType$1 = {}));

  var MEDIA_SCHEMA_PREFIX = 'iglu:com.snowplowanalytics.snowplow.media/';
  var MEDIA_SCHEMA_SUFFIX = '/jsonschema/1-0-0';
  // NOTE: The player schema has a different vendor than the other media schemas because it builds on an older version of the same schema. Versions 3.12 to 3.13.1 of the media plugin used a conflicting schema URI which has since been removed from Iglu Central.
  var MEDIA_PLAYER_SCHEMA = 'iglu:com.snowplowanalytics.snowplow/media_player/jsonschema/2-0-0';
  var MEDIA_SESSION_SCHEMA = MEDIA_SCHEMA_PREFIX + 'session' + MEDIA_SCHEMA_SUFFIX;
  var MEDIA_AD_SCHEMA = MEDIA_SCHEMA_PREFIX + 'ad' + MEDIA_SCHEMA_SUFFIX;
  var MEDIA_AD_BREAK_SCHEMA = MEDIA_SCHEMA_PREFIX + 'ad_break' + MEDIA_SCHEMA_SUFFIX;
  function eventNameForEventType(eventType) {
      /// ad first quartile, midpoint, and third quartile events share the same schema
      switch (eventType) {
          case MediaEventType$1.AdFirstQuartile:
              return 'ad_quartile';
          case MediaEventType$1.AdMidpoint:
              return 'ad_quartile';
          case MediaEventType$1.AdThirdQuartile:
              return 'ad_quartile';
      }
      return eventType;
  }
  function getMediaEventSchema(eventType) {
      var eventName = eventNameForEventType(eventType);
      return MEDIA_SCHEMA_PREFIX + eventName + '_event' + MEDIA_SCHEMA_SUFFIX;
  }

  function buildMediaPlayerEvent(event) {
      var _a;
      return {
          schema: getMediaEventSchema(event.type),
          data: removeEmptyProperties((_a = event.eventBody) !== null && _a !== void 0 ? _a : {}),
      };
  }
  function buildMediaPlayerEntity(mediaPlayer) {
      return {
          schema: MEDIA_PLAYER_SCHEMA,
          data: removeEmptyProperties(mediaPlayer),
      };
  }
  function buildMediaSessionEntity(session) {
      return {
          schema: MEDIA_SESSION_SCHEMA,
          data: removeEmptyProperties(session),
      };
  }
  function buildMediaAdEntity(ad) {
      return {
          schema: MEDIA_AD_SCHEMA,
          data: removeEmptyProperties(ad),
      };
  }
  function buildMediaAdBreakEntity(adBreak) {
      return {
          schema: MEDIA_AD_BREAK_SCHEMA,
          data: removeEmptyProperties(adBreak),
      };
  }
  /**
   * Returns a copy of a JSON with undefined and null properties removed
   *
   * @param event - Object to clean
   * @returns A cleaned copy of eventJson
   */
  function removeEmptyProperties(event) {
      var ret = {};
      for (var k in event) {
          if (event[k] != null) {
              ret[k] = event[k];
          }
      }
      return ret;
  }

  /** Keeps track of the ad and ad break entities and updates them according to tracked events. */
  var MediaAdTracking = /** @class */ (function () {
      function MediaAdTracking() {
          this.podPosition = 0;
      }
      MediaAdTracking.prototype.updateForThisEvent = function (eventType, player, ad, adBreak) {
          if (eventType == MediaEventType$1.AdStart) {
              this.ad = undefined;
              this.podPosition++;
          }
          else if (eventType == MediaEventType$1.AdBreakStart) {
              this.adBreak = undefined;
              this.podPosition = 0;
          }
          if (ad !== undefined) {
              var position = { podPosition: this.podPosition > 0 ? this.podPosition : undefined };
              if (this.ad !== undefined) {
                  this.ad = __assign(__assign(__assign({}, this.ad), position), ad);
              }
              else {
                  this.ad = __assign(__assign({}, position), ad);
              }
          }
          if (adBreak !== undefined) {
              var startTime = { startTime: player.currentTime };
              if (this.adBreak !== undefined) {
                  this.adBreak = __assign(__assign(__assign({}, startTime), this.adBreak), adBreak);
              }
              else {
                  this.adBreak = __assign(__assign({}, startTime), adBreak);
              }
          }
      };
      MediaAdTracking.prototype.updateForNextEvent = function (eventType) {
          if (eventType == MediaEventType$1.AdBreakEnd) {
              this.adBreak = undefined;
              this.podPosition = 0;
          }
          if (eventType == MediaEventType$1.AdComplete || eventType == MediaEventType$1.AdSkip) {
              this.ad = undefined;
          }
      };
      MediaAdTracking.prototype.getContext = function () {
          var context = [];
          if (this.ad !== undefined) {
              context.push(buildMediaAdEntity(this.ad));
          }
          if (this.adBreak !== undefined) {
              context.push(buildMediaAdBreakEntity(this.adBreak));
          }
          return context;
      };
      return MediaAdTracking;
  }());

  /**
   * This class filters out repeated events that are sent by the media player.
   * This applies to seek and volume change events.
   */
  var RepeatedEventFilter = /** @class */ (function () {
      function RepeatedEventFilter(configuration) {
          var _this = this;
          var _a;
          this.aggregateEventsWithOrder = {};
          this.eventsToAggregate = {};
          var allFiltersEnabled = configuration === undefined || configuration === true;
          if (allFiltersEnabled || (typeof configuration === 'object' && configuration.seekEvents !== false)) {
              this.aggregateEventsWithOrder[getMediaEventSchema(MediaEventType$1.SeekStart)] = true;
              this.aggregateEventsWithOrder[getMediaEventSchema(MediaEventType$1.SeekEnd)] = false;
          }
          if (allFiltersEnabled || (typeof configuration === 'object' && configuration.volumeChangeEvents !== false)) {
              this.aggregateEventsWithOrder[getMediaEventSchema(MediaEventType$1.VolumeChange)] = false;
          }
          this.flushTimeoutMs = (_a = (typeof configuration === 'object' ? configuration.flushTimeoutMs : undefined)) !== null && _a !== void 0 ? _a : 5000;
          Object.keys(this.aggregateEventsWithOrder).forEach(function (schema) {
              _this.eventsToAggregate[schema] = [];
          });
      }
      RepeatedEventFilter.prototype.trackFilteredEvents = function (events, trackEvent) {
          var _this = this;
          var startFlushTimeout = false;
          events.forEach(function (_a) {
              var event = _a.event, context = _a.context;
              if (_this.eventsToAggregate[event.schema] !== undefined) {
                  startFlushTimeout = true;
                  _this.eventsToAggregate[event.schema].push(function () { return trackEvent({ event: event, context: context }); });
              }
              else {
                  startFlushTimeout = false;
                  // flush any events waiting
                  _this.flush();
                  trackEvent({ event: event, context: context });
              }
          });
          if (startFlushTimeout && this.flushTimeout === undefined) {
              this.setFlushTimeout();
          }
      };
      RepeatedEventFilter.prototype.flush = function () {
          var _this = this;
          this.clearFlushTimeout();
          Object.keys(this.eventsToAggregate).forEach(function (schema) {
              var eventsToAggregate = _this.eventsToAggregate[schema];
              if (eventsToAggregate.length > 0) {
                  if (_this.aggregateEventsWithOrder[schema]) {
                      eventsToAggregate[0]();
                  }
                  else {
                      eventsToAggregate[eventsToAggregate.length - 1]();
                  }
                  _this.eventsToAggregate[schema] = [];
              }
          });
      };
      RepeatedEventFilter.prototype.clearFlushTimeout = function () {
          if (this.flushTimeout !== undefined) {
              clearTimeout(this.flushTimeout);
              this.flushTimeout = undefined;
          }
      };
      RepeatedEventFilter.prototype.setFlushTimeout = function () {
          var _this = this;
          this.clearFlushTimeout();
          this.flushTimeout = window.setTimeout(function () { return _this.flush(); }, this.flushTimeoutMs);
      };
      return RepeatedEventFilter;
  }());

  /**
   * Manages the state and built-in entities for a media tracking that starts when a user
   * call `startMediaTracking` and ends with `endMediaTracking`.
   *
   * It updates the internal state for each tracked event and returns context entities with
   * properties updated based on the internal state.
   */
  var MediaTracking = /** @class */ (function () {
      function MediaTracking(id, player, session, pingInterval, boundaries, captureEvents, updatePageActivityWhilePlaying, filterRepeatedEvents, context) {
          /// List of boundaries for which percent progress events were already sent to avoid sending again.
          this.sentBoundaries = [];
          /// State for the media player context entity that is updated as new events are tracked.
          this.player = {
              currentTime: 0,
              paused: true,
              ended: false,
          };
          /// Manages ad entities.
          this.adTracking = new MediaAdTracking();
          this.id = id;
          this.updatePlayer(player);
          this.session = session;
          this.pingInterval = pingInterval;
          this.boundaries = boundaries;
          this.captureEvents = captureEvents;
          this.updatePageActivityWhilePlaying = updatePageActivityWhilePlaying;
          this.customContext = context;
          this.repeatedEventFilter = new RepeatedEventFilter(filterRepeatedEvents);
          // validate event names in the captureEvents list
          captureEvents === null || captureEvents === void 0 ? void 0 : captureEvents.forEach(function (eventType) {
              if (!Object.values(MediaEventType$1).includes(eventType)) {
                  LOG$1.warn('Unknown media event ' + eventType);
              }
          });
      }
      /**
       * Called when user calls `endMediaTracking()`.
       */
      MediaTracking.prototype.flushAndStop = function () {
          var _a;
          (_a = this.pingInterval) === null || _a === void 0 ? void 0 : _a.clear();
          this.repeatedEventFilter.flush();
      };
      /**
       * Updates the internal state given the new event or new media player info and returns events to track.
       * @param eventType Type of the event tracked or undefined when only updating player properties.
       * @param player Updates to the media player stored entity.
       * @param ad Updates to the ad entity.
       * @param adBreak Updates to the ad break entity.
       * @returns List of events with entities to track.
       */
      MediaTracking.prototype.update = function (trackEvent, mediaEvent, customEvent, player, ad, adBreak) {
          var _a, _b;
          // update state
          this.updatePlayer(player);
          if (mediaEvent !== undefined) {
              this.adTracking.updateForThisEvent(mediaEvent.type, this.player, ad, adBreak);
          }
          (_a = this.session) === null || _a === void 0 ? void 0 : _a.update(mediaEvent === null || mediaEvent === void 0 ? void 0 : mediaEvent.type, this.player, this.adTracking.adBreak);
          (_b = this.pingInterval) === null || _b === void 0 ? void 0 : _b.update(this.player);
          // build context entities
          var context = [buildMediaPlayerEntity(this.player)];
          if (this.session !== undefined) {
              context.push(this.session.getContext());
          }
          if (this.customContext) {
              context = context.concat(resolveDynamicContext(this.customContext, mediaEvent !== null && mediaEvent !== void 0 ? mediaEvent : customEvent));
          }
          context = context.concat(this.adTracking.getContext());
          // build event types to track
          var mediaEventsToTrack = [];
          if (mediaEvent !== undefined && this.shouldTrackEvent(mediaEvent.type)) {
              mediaEventsToTrack.push(mediaEvent);
          }
          if (this.shouldSendPercentProgress()) {
              mediaEventsToTrack.push({
                  type: MediaEventType$1.PercentProgress,
                  eventBody: { percentProgress: this.getPercentProgress() },
              });
          }
          // update state for events after this one
          if (mediaEvent !== undefined) {
              this.adTracking.updateForNextEvent(mediaEvent.type);
          }
          var eventsToTrack = mediaEventsToTrack.map(function (event) {
              return { event: buildMediaPlayerEvent(event), context: context };
          });
          if (customEvent !== undefined) {
              eventsToTrack.push({ event: customEvent, context: context });
          }
          this.repeatedEventFilter.trackFilteredEvents(eventsToTrack, trackEvent);
      };
      MediaTracking.prototype.shouldUpdatePageActivity = function () {
          var _a;
          return ((_a = this.updatePageActivityWhilePlaying) !== null && _a !== void 0 ? _a : true) && !this.player.paused;
      };
      MediaTracking.prototype.updatePlayer = function (player) {
          if (player !== undefined) {
              this.player = __assign(__assign({}, this.player), player);
          }
      };
      MediaTracking.prototype.shouldSendPercentProgress = function () {
          var percentProgress = this.getPercentProgress();
          if (this.boundaries === undefined || percentProgress === undefined || this.player.paused) {
              return false;
          }
          var boundaries = this.boundaries.filter(function (b) { return b <= (percentProgress !== null && percentProgress !== void 0 ? percentProgress : 0); });
          if (boundaries.length == 0) {
              return false;
          }
          var boundary = Math.max.apply(Math, boundaries);
          if (!this.sentBoundaries.includes(boundary)) {
              this.sentBoundaries.push(boundary);
              return true;
          }
          return false;
      };
      MediaTracking.prototype.shouldTrackEvent = function (eventType) {
          return this.captureEvents === undefined || this.captureEvents.includes(eventType);
      };
      MediaTracking.prototype.getPercentProgress = function () {
          var _a;
          if (this.player.duration === null || this.player.duration === undefined || this.player.duration == 0) {
              return undefined;
          }
          return Math.floor((((_a = this.player.currentTime) !== null && _a !== void 0 ? _a : 0) / this.player.duration) * 100);
      };
      return MediaTracking;
  }());

  /**
   * Default ping interval in seconds.
   * Changing the setting is a breaking change for downstream models that work with this setting.
   * If a custom interval is used, it is present in the session context entity.
   **/
  var DEFAULT_PING_INTERVAL = 30;
  /** Manages the timer for firing the media ping events. */
  var MediaPingInterval = /** @class */ (function () {
      function MediaPingInterval(pingIntervalSeconds, maxPausedPings, trackPing) {
          var _this = this;
          this.numPausedPings = 0;
          this.maxPausedPings = 1;
          if (maxPausedPings !== undefined) {
              this.maxPausedPings = maxPausedPings;
          }
          this.interval = setInterval(function () {
              if (!_this.isPaused() || _this.numPausedPings < _this.maxPausedPings) {
                  if (_this.isPaused()) {
                      _this.numPausedPings++;
                  }
                  trackPing();
              }
          }, (pingIntervalSeconds !== null && pingIntervalSeconds !== void 0 ? pingIntervalSeconds : DEFAULT_PING_INTERVAL) * 1000);
      }
      MediaPingInterval.prototype.update = function (player) {
          this.paused = player.paused;
          if (!this.paused) {
              this.numPausedPings = 0;
          }
      };
      MediaPingInterval.prototype.clear = function () {
          if (this.interval !== undefined) {
              clearInterval(this.interval);
              this.interval = undefined;
          }
      };
      MediaPingInterval.prototype.isPaused = function () {
          return this.paused === true;
      };
      return MediaPingInterval;
  }());

  var adStartTypes = [MediaEventType$1.AdStart, MediaEventType$1.AdResume];
  var adProgressTypes = [
      MediaEventType$1.AdClick,
      MediaEventType$1.AdFirstQuartile,
      MediaEventType$1.AdMidpoint,
      MediaEventType$1.AdThirdQuartile,
  ];
  var adEndTypes = [MediaEventType$1.AdComplete, MediaEventType$1.AdSkip, MediaEventType$1.AdPause];
  var bufferingEndTypes = [MediaEventType$1.BufferEnd, MediaEventType$1.Play];
  /** Calculates the average playback rate based on measurements of the rate over partial durations */
  var AveragePlaybackRateCalculator = /** @class */ (function () {
      function AveragePlaybackRateCalculator() {
          this.durationWithPlaybackRate = 0;
          this.duration = 0;
      }
      AveragePlaybackRateCalculator.prototype.add = function (rate, duration) {
          this.durationWithPlaybackRate += rate * duration;
          this.duration += duration;
      };
      AveragePlaybackRateCalculator.prototype.get = function () {
          return this.duration > 0 ? this.durationWithPlaybackRate / this.duration : undefined;
      };
      return AveragePlaybackRateCalculator;
  }());
  /**
   * Calculates statistics in the media player session as events are tracked.
   */
  var MediaSessionTrackingStats = /** @class */ (function () {
      function MediaSessionTrackingStats() {
          /// time for which ads were playing
          this.adPlaybackDuration = 0;
          /// time for which the content was playing
          this.playbackDuration = 0;
          /// time for which the content was playing on mute
          this.playbackDurationMuted = 0;
          /// average playback rate calculator
          this.avgPlaybackRate = new AveragePlaybackRateCalculator();
          /// time for which the playback was paused
          this.pausedDuration = 0;
          /// number of ads
          this.ads = 0;
          /// number of ad breaks
          this.adBreaks = 0;
          /// number of ad skip events
          this.adsSkipped = 0;
          /// number of ad click events
          this.adsClicked = 0;
          /// sum of time durations between the buffer start event and end of buffering
          this.bufferingDuration = 0;
          /// set of seconds in content time that were played used to calculate the content watched duration
          this.playedSeconds = new Set();
      }
      /// Update stats given a new event.
      MediaSessionTrackingStats.prototype.update = function (eventType, player, adBreak) {
          var _a;
          var log = {
              time: new Date().getTime() / 1000,
              contentTime: player.currentTime,
              eventType: eventType,
              playbackRate: player.playbackRate,
              paused: player.paused,
              muted: player.muted,
              linearAd: ((_a = adBreak === null || adBreak === void 0 ? void 0 : adBreak.breakType) !== null && _a !== void 0 ? _a : MediaAdBreakType$1.Linear) == MediaAdBreakType$1.Linear,
          };
          this.updateDurationStats(log);
          this.updateAdStats(log);
          this.updateBufferingStats(log);
          this.lastLog = log;
      };
      /// Produce part of the media session entity with the stats.
      MediaSessionTrackingStats.prototype.toSessionContextEntity = function () {
          return {
              timePaused: this.pausedDuration > 0 ? this.round(this.pausedDuration) : undefined,
              timePlayed: this.playbackDuration > 0 ? this.round(this.playbackDuration) : undefined,
              timePlayedMuted: this.playbackDurationMuted > 0 ? this.round(this.playbackDurationMuted) : undefined,
              timeSpentAds: this.adPlaybackDuration > 0 ? this.round(this.adPlaybackDuration) : undefined,
              timeBuffering: this.bufferingDuration > 0 ? this.round(this.bufferingDuration) : undefined,
              ads: this.ads > 0 ? this.ads : undefined,
              adBreaks: this.adBreaks > 0 ? this.adBreaks : undefined,
              adsSkipped: this.adsSkipped > 0 ? this.adsSkipped : undefined,
              adsClicked: this.adsClicked > 0 ? this.adsClicked : undefined,
              avgPlaybackRate: this.round(this.avgPlaybackRate.get()),
              contentWatched: this.playedSeconds.size > 0 ? this.playedSeconds.size : undefined,
          };
      };
      MediaSessionTrackingStats.prototype.updateDurationStats = function (log) {
          var _a;
          // if ad was playing until now and it was a linear ad, don't add the duration stats
          var wasPlayingAd = this.lastAdUpdateAt !== undefined;
          var shouldCountStats = (_a = (!wasPlayingAd || !log.linearAd)) !== null && _a !== void 0 ? _a : true;
          if (!shouldCountStats) {
              return;
          }
          if (this.lastLog !== undefined) {
              // add the time diff since last event to duration stats
              var duration = log.time - this.lastLog.time;
              if (this.lastLog.paused) {
                  this.pausedDuration += duration;
              }
              else {
                  this.playbackDuration += duration;
                  if (this.lastLog.playbackRate !== undefined) {
                      this.avgPlaybackRate.add(this.lastLog.playbackRate, duration);
                  }
                  if (this.lastLog.muted) {
                      this.playbackDurationMuted += duration;
                  }
                  if (!log.paused) {
                      for (var i = Math.floor(this.lastLog.contentTime); i < log.contentTime; i++) {
                          this.playedSeconds.add(i);
                      }
                  }
              }
          }
          if (!log.paused) {
              this.playedSeconds.add(Math.floor(log.contentTime));
          }
      };
      MediaSessionTrackingStats.prototype.updateAdStats = function (log) {
          // only works with ad event types
          if (log.eventType === undefined) {
              return;
          }
          // count ad actions
          if (log.eventType == MediaEventType$1.AdBreakStart) {
              this.adBreaks++;
          }
          else if (log.eventType == MediaEventType$1.AdStart) {
              this.ads++;
          }
          else if (log.eventType == MediaEventType$1.AdSkip) {
              this.adsSkipped++;
          }
          else if (log.eventType == MediaEventType$1.AdClick) {
              this.adsClicked++;
          }
          // update ad playback duration
          if (this.lastAdUpdateAt === undefined) {
              if (adStartTypes.includes(log.eventType)) {
                  this.lastAdUpdateAt = log.time;
              }
          }
          else if (adProgressTypes.includes(log.eventType)) {
              this.adPlaybackDuration += log.time - this.lastAdUpdateAt;
              this.lastAdUpdateAt = log.time;
          }
          else if (adEndTypes.includes(log.eventType)) {
              this.adPlaybackDuration += log.time - this.lastAdUpdateAt;
              this.lastAdUpdateAt = undefined;
          }
      };
      MediaSessionTrackingStats.prototype.updateBufferingStats = function (log) {
          if (log.eventType == MediaEventType$1.BufferStart) {
              this.bufferingStartedAt = log.time;
              this.bufferingStartTime = log.contentTime;
          }
          else if (this.bufferingStartedAt !== undefined) {
              // Either the playback moved or BufferEnd or Play events were tracked
              if ((log.contentTime != this.bufferingStartTime && !log.paused) ||
                  (log.eventType !== undefined && bufferingEndTypes.includes(log.eventType))) {
                  this.bufferingDuration += log.time - this.bufferingStartedAt;
                  this.bufferingStartedAt = undefined;
                  this.bufferingStartTime = undefined;
              }
              else {
                  this.bufferingDuration += log.time - this.bufferingStartedAt;
                  this.bufferingStartedAt = log.time;
              }
          }
      };
      MediaSessionTrackingStats.prototype.round = function (n) {
          if (n === undefined) {
              return undefined;
          }
          return Math.round(n * 1000) / 1000;
      };
      return MediaSessionTrackingStats;
  }());

  /**
   * Manages the media player session that is optionally added as a context entity
   * in media tracking.
   */
  var MediaSessionTracking = /** @class */ (function () {
      function MediaSessionTracking(id, startedAt, pingInterval) {
          this.stats = new MediaSessionTrackingStats();
          this.id = id;
          this.pingInterval = pingInterval;
          this.startedAt = startedAt !== null && startedAt !== void 0 ? startedAt : new Date();
      }
      MediaSessionTracking.prototype.update = function (eventType, player, adBreak) {
          this.stats.update(eventType, player, adBreak);
      };
      MediaSessionTracking.prototype.getContext = function () {
          return buildMediaSessionEntity(__assign({ mediaSessionId: this.id, startedAt: this.startedAt, pingInterval: this.pingInterval }, this.stats.toSessionContextEntity()));
      };
      return MediaSessionTracking;
  }());

  var _trackers = {};
  var _context = {};
  /**
   * Adds media tracking
   */
  function SnowplowMediaPlugin() {
      var trackerId;
      return {
          activateBrowserPlugin: function (tracker) {
              trackerId = tracker.id;
              _trackers[trackerId] = tracker;
              _context[trackerId] = [];
          },
          contexts: function () {
              return _context[trackerId] || [];
          },
      };
  }
  var activeMedias = {};
  /**
   * Starts media tracking for a single media content tracked in a media player.
   * The tracking instance is uniquely identified by a given ID.
   * All subsequent media track calls will be processed within this media tracking if given the same ID.
   *
   * @param config Configuration for setting up media tracking
   * @param trackers The tracker identifiers which ping events will be sent to
   */
  function startMediaTracking(config, trackers) {
      var _a, _b, _c;
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var pingInterval = typeof config.pings === 'boolean' ? undefined : (_a = config.pings) === null || _a === void 0 ? void 0 : _a.pingInterval;
      var maxPausedPings = typeof config.pings === 'boolean' ? undefined : (_b = config.pings) === null || _b === void 0 ? void 0 : _b.maxPausedPings;
      var pings = config.pings === false || config.pings === undefined
          ? undefined
          : new MediaPingInterval(pingInterval, maxPausedPings, function () {
              track({ mediaEvent: { type: MediaEventType$1.Ping } }, { id: config.id }, trackers);
          });
      var sessionTracking = config.session === false ? undefined : new MediaSessionTracking(config.id, (_c = config.session) === null || _c === void 0 ? void 0 : _c.startedAt, pingInterval);
      var mediaTracking = new MediaTracking(config.id, config.player, sessionTracking, pings, config.boundaries, config.captureEvents, config.updatePageActivityWhilePlaying, config.filterOutRepeatedEvents, config.context);
      activeMedias[mediaTracking.id] = mediaTracking;
  }
  /**
   * Ends media tracking with the given ID if previously started.
   * Clears local state for the media tracking and sends any events waiting to be sent.
   *
   * @param configuration Configuration with the media tracking ID
   */
  function endMediaTracking(configuration) {
      if (activeMedias[configuration.id]) {
          activeMedias[configuration.id].flushAndStop();
          delete activeMedias[configuration.id];
      }
  }
  /**
   * Updates stored attributes of the media tracking such as the current playback.
   * Use this function to continually update the player attributes so that they
   * can be sent in the background ping events.
   *
   * @param args The attributes for the media entities
   * @param trackers The tracker identifiers which any resulting event will be sent to
   */
  function updateMediaTracking(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({}, args, trackers);
  }
  /**
   * Tracks a media player ready event that is fired when the media tracking is successfully
   * attached to the player and can track events.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaReady(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.Ready } }, args, trackers);
  }
  /**
   * Tracks a media player play event sent when the player changes state to playing from
   * previously being paused.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaPlay(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.Play } }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { paused: false }) }), trackers);
  }
  /**
   * Tracks a media player pause event sent when the user pauses the playback.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaPause(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.Pause } }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { paused: true }) }), trackers);
  }
  /**
   * Tracks a media player end event sent when playback stops when end of the media
   * is reached or because no further data is available.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaEnd(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.End } }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { ended: true, paused: true }) }), trackers);
  }
  /**
   * Tracks a media player seek end event sent when a seek operation completes.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaSeekEnd(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.SeekEnd } }, args, trackers);
  }
  /**
   * Tracks a media player playback rate change event sent when the playback rate has changed.
   *
   * If not passed here, the previous rate is taken from the last setting in media player.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaPlaybackRateChange(args, trackers) {
      var _a;
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var previousRate = args.previousRate, newRate = args.newRate;
      track({
          mediaEvent: {
              type: MediaEventType$1.PlaybackRateChange,
              eventBody: {
                  previousRate: previousRate !== null && previousRate !== void 0 ? previousRate : (_a = getMedia(args.id)) === null || _a === void 0 ? void 0 : _a.player.playbackRate,
                  newRate: newRate,
              },
          },
      }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { playbackRate: newRate }) }), trackers);
  }
  /**
   * Tracks a media player volume change event sent when the volume has changed.
   *
   * If not passed here, the previous volume is taken from the last setting in media player.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaVolumeChange(args, trackers) {
      var _a;
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var previousVolume = args.previousVolume, newVolume = args.newVolume;
      track({
          mediaEvent: {
              type: MediaEventType$1.VolumeChange,
              eventBody: {
                  previousVolume: previousVolume !== null && previousVolume !== void 0 ? previousVolume : (_a = getMedia(args.id)) === null || _a === void 0 ? void 0 : _a.player.volume,
                  newVolume: newVolume,
              },
          },
      }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { volume: newVolume }) }), trackers);
  }
  /**
   * Tracks a media player fullscreen change event fired immediately after
   * the browser switches into or out of full-screen mode.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaFullscreenChange(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var fullscreen = args.fullscreen;
      track({
          mediaEvent: {
              type: MediaEventType$1.FullscreenChange,
              eventBody: { fullscreen: fullscreen },
          },
      }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { fullscreen: fullscreen }) }), trackers);
  }
  /**
   * Tracks a media player picture-in-picture change event fired immediately
   * after the browser switches into or out of picture-in-picture mode.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaPictureInPictureChange(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var pictureInPicture = args.pictureInPicture;
      track({
          mediaEvent: {
              type: MediaEventType$1.PictureInPictureChange,
              eventBody: { pictureInPicture: pictureInPicture },
          },
      }, __assign(__assign({}, args), { player: __assign(__assign({}, args.player), { pictureInPicture: pictureInPicture }) }), trackers);
  }
  /**
   * Tracks a media player buffering start event fired when the player goes
   * into the buffering state and begins to buffer content.
   *
   * End of buffering can be tracked using `trackMediaBufferEnd` or `trackMediaPlay` or
   * by updating the `currentTime` property.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaBufferStart(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.BufferStart } }, args, trackers);
  }
  /**
   * Tracks a media player buffering end event fired when the the player
   * finishes buffering content and resumes playback.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaBufferEnd(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      track({ mediaEvent: { type: MediaEventType$1.BufferEnd } }, args, trackers);
  }
  /**
   * Tracks a media player error event tracked when the resource could not be loaded due to an error.
   *
   * @param args The attributes for the media player event and entities
   * @param trackers The tracker identifiers which the event will be sent to
   */
  function trackMediaError(args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var errorCode = args.errorCode, errorName = args.errorName, errorDescription = args.errorDescription;
      track({
          mediaEvent: {
              type: MediaEventType$1.Error,
              eventBody: { errorCode: errorCode, errorName: errorName, errorDescription: errorDescription },
          },
      }, args, trackers);
  }
  function getMedia(id) {
      if (activeMedias[id] === undefined) {
          LOG$1.error("Media tracking ".concat(id, " not started."));
          return undefined;
      }
      return activeMedias[id];
  }
  function track(event, args, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var _a = args.context, context = _a === void 0 ? [] : _a, timestamp = args.timestamp, player = args.player, ad = args.ad, adBreak = args.adBreak, id = args.id;
      var mediaEvent = event.mediaEvent, customEvent = event.customEvent;
      var mediaTracking = getMedia(id);
      if (mediaTracking === undefined) {
          return;
      }
      var trackEvent = function (event) {
          dispatchToTrackersInCollection(trackers, _trackers, function (t) {
              var _a;
              t.core.track(buildSelfDescribingEvent(event), ((_a = event.context) !== null && _a !== void 0 ? _a : []).concat(resolveDynamicContext(context)), timestamp);
          });
      };
      mediaTracking.update(trackEvent, mediaEvent, customEvent, player, ad, adBreak);
      // Update page activity in order to keep sending page pings if needed
      if (mediaTracking.shouldUpdatePageActivity()) {
          dispatchToTrackersInCollection(trackers, _trackers, function (t) {
              t.updatePageActivity();
          });
      }
  }

  /** Type of media player event */
  var MediaEventType;
  (function (MediaEventType) {
      // Controlling the playback
      /** Media player event fired when the media tracking is successfully attached to the player and can track events. */
      MediaEventType["Ready"] = "ready";
      /** Media player event sent when the player changes state to playing from previously being paused. */
      MediaEventType["Play"] = "play";
      /** Media player event sent when the user pauses the playback. */
      MediaEventType["Pause"] = "pause";
      /** Media player event sent when playback stops when end of the media is reached or because no further data is available. */
      MediaEventType["End"] = "end";
      /** Media player event sent when a seek operation begins. */
      MediaEventType["SeekStart"] = "seek_start";
      /** Media player event sent when a seek operation completes. */
      MediaEventType["SeekEnd"] = "seek_end";
      // Changes in playback settings
      /** Media player event sent when the playback rate has changed. */
      MediaEventType["PlaybackRateChange"] = "playback_rate_change";
      /** Media player event sent when the volume has changed. */
      MediaEventType["VolumeChange"] = "volume_change";
      /** Media player event fired immediately after the browser switches into or out of full-screen mode. */
      MediaEventType["FullscreenChange"] = "fullscreen_change";
      /** Media player event fired immediately after the browser switches into or out of picture-in-picture mode. */
      MediaEventType["PictureInPictureChange"] = "picture_in_picture_change";
      // Tracking playback progress
      /** Media player event fired periodically during main content playback, regardless of other API events that have been sent. */
      MediaEventType["Ping"] = "ping";
      /** Media player event fired when a percentage boundary set in options.boundaries is reached */
      MediaEventType["PercentProgress"] = "percent_progress";
      // Ad events
      /** Media player event that signals the start of an ad break. */
      MediaEventType["AdBreakStart"] = "ad_break_start";
      /** Media player event that signals the end of an ad break. */
      MediaEventType["AdBreakEnd"] = "ad_break_end";
      /** Media player event that signals the start of an ad. */
      MediaEventType["AdStart"] = "ad_start";
      /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdFirstQuartile"] = "ad_first_quartile";
      /** Media player event fired when a midpoint of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdMidpoint"] = "ad_midpoint";
      /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
      MediaEventType["AdThirdQuartile"] = "ad_third_quartile";
      /** Media player event that signals the ad creative was played to the end at normal speed. */
      MediaEventType["AdComplete"] = "ad_complete";
      /** Media player event fired when the user activated a skip control to skip the ad creative. */
      MediaEventType["AdSkip"] = "ad_skip";
      /** Media player event fired when the user clicked on the ad. */
      MediaEventType["AdClick"] = "ad_click";
      /** Media player event fired when the user clicked the pause control and stopped the ad creative. */
      MediaEventType["AdPause"] = "ad_pause";
      /** Media player event fired when the user resumed playing the ad creative after it had been stopped or paused. */
      MediaEventType["AdResume"] = "ad_resume";
      // Data quality events
      /** Media player event fired when the player goes into the buffering state and begins to buffer content. */
      MediaEventType["BufferStart"] = "buffer_start";
      /** Media player event fired when the the player finishes buffering content and resumes playback. */
      MediaEventType["BufferEnd"] = "buffer_end";
      /** Media player event tracked when the video playback quality changes automatically. */
      MediaEventType["QualityChange"] = "quality_change";
      /** Media player event tracked when the resource could not be loaded due to an error.  */
      MediaEventType["Error"] = "error";
  })(MediaEventType || (MediaEventType = {}));
  /** Type of media content. */
  var MediaType;
  (function (MediaType) {
      /** Audio content. */
      MediaType["Audio"] = "audio";
      /** Video content. */
      MediaType["Video"] = "video";
  })(MediaType || (MediaType = {}));
  /** Type of ads within the break */
  var MediaAdBreakType;
  (function (MediaAdBreakType) {
      /** Take full control of the video for a period of time. */
      MediaAdBreakType["Linear"] = "linear";
      /** Run concurrently to the video. */
      MediaAdBreakType["NonLinear"] = "nonlinear";
      /** Accompany the video but placed outside the player. */
      MediaAdBreakType["Companion"] = "companion";
  })(MediaAdBreakType || (MediaAdBreakType = {}));

  function buildHTMLMediaElementEntity(el) {
      var _a, _b;
      var data = {
          // htmlId is a required property in the schema, but may not be present if
          // the user provided the element themselves
          htmlId: el.id || '',
          mediaType: el.tagName,
          autoPlay: el.autoplay,
          buffered: timeRangesToObjectArray(el.buffered),
          controls: el.controls,
          currentSrc: el.currentSrc,
          defaultMuted: el.defaultMuted || false,
          defaultPlaybackRate: el.defaultPlaybackRate,
          error: el.error ? { code: (_a = el.error) === null || _a === void 0 ? void 0 : _a.code, message: (_b = el.error) === null || _b === void 0 ? void 0 : _b.message } : null,
          networkState: NETWORK_STATE[el.networkState],
          preload: el.preload,
          readyState: READY_STATE[el.readyState],
          seekable: timeRangesToObjectArray(el.seekable),
          seeking: el.seeking,
          src: dataUrlHandler(el.src || el.currentSrc),
          textTracks: textTrackListToJson(el.textTracks),
          fileExtension: getUriFileExtension(el.currentSrc),
          fullscreen: isFullScreen(el),
          pictureInPicture: document.pictureInPictureElement === el,
      };
      if (el.disableRemotePlayback)
          data.disableRemotePlayback = el.disableRemotePlayback;
      if (el.crossOrigin)
          data.crossOrigin = el.crossOrigin;
      return {
          schema: 'iglu:org.whatwg/media_element/jsonschema/1-0-0',
          data: data,
      };
  }
  function buildHTMLVideoElementEntity(el) {
      var data = {
          poster: el.poster,
          videoHeight: el.videoHeight,
          videoWidth: el.videoWidth,
      };
      if (el.hasAttribute('autopictureinpicture'))
          data.autoPictureInPicture = true;
      if (el.disablePictureInPicture)
          data.disablePictureInPicture = el.disablePictureInPicture;
      return {
          schema: 'iglu:org.whatwg/video_element/jsonschema/1-0-0',
          data: data,
      };
  }

  function updatePlayer(el) {
      var common = {
          currentTime: el.currentTime || 0,
          duration: getDuration(el),
          ended: el.ended,
          livestream: el.duration === Infinity,
          loop: el.loop,
          mediaType: isHtmlVideoElement(el) ? MediaType.Video : MediaType.Audio,
          muted: el.muted,
          paused: el.paused,
          volume: parseVolume(el.volume),
          playbackRate: el.playbackRate,
      };
      var video = el instanceof HTMLVideoElement
          ? {
              fullscreen: isFullScreen(el),
              pictureInPicture: document.pictureInPictureElement === el,
          }
          : {};
      return __assign(__assign({}, common), video);
  }
  function htmlContext(el) {
      var context = [function () { return buildHTMLMediaElementEntity(el); }];
      if (el instanceof HTMLVideoElement) {
          context.push(function () { return buildHTMLVideoElementEntity(el); });
      }
      return context;
  }
  function setUpListeners(config) {
      var _a;
      var id = config.id, video = config.video, label = config.label;
      startMediaTracking(__assign(__assign({}, config), { id: id, player: __assign({ label: label }, updatePlayer(video)), context: ((_a = config.context) !== null && _a !== void 0 ? _a : []).concat(htmlContext(video)) }));
      // If metadata is already loaded, we can track the ready event immediately
      if (video.readyState > 0) {
          trackMediaReady({ id: id, player: updatePlayer(video) });
      }
      else {
          video.addEventListener('loadedmetadata', function () {
              trackMediaReady({ id: id, player: updatePlayer(video) });
          });
      }
      addVideoEventListeners(video, id);
  }
  function addVideoEventListeners(video, id) {
      var isWaiting = false;
      video.addEventListener('play', function () { return trackMediaPlay({ id: id, player: updatePlayer(video) }); });
      video.addEventListener('pause', function () { return trackMediaPause({ id: id, player: updatePlayer(video) }); });
      video.addEventListener('ended', function () { return trackMediaEnd({ id: id, player: updatePlayer(video) }); });
      video.addEventListener('seeked', function () { return trackMediaSeekEnd({ id: id, player: updatePlayer(video) }); });
      video.addEventListener('ratechange', function () {
          return trackMediaPlaybackRateChange({ id: id, player: updatePlayer(video), newRate: video.playbackRate });
      });
      video.addEventListener('volumechange', function () {
          trackMediaVolumeChange({ id: id, player: updatePlayer(video), newVolume: parseVolume(video.volume) });
      });
      video.addEventListener('fullscreenchange', function () {
          trackMediaFullscreenChange({
              id: id,
              player: updatePlayer(video),
              fullscreen: document.fullscreenElement === video,
          });
      });
      video.addEventListener('enterpictureinpicture', function () {
          trackMediaPictureInPictureChange({
              id: id,
              player: updatePlayer(video),
              pictureInPicture: true,
          });
      });
      video.addEventListener('leavepictureinpicture', function () {
          trackMediaPictureInPictureChange({
              id: id,
              player: updatePlayer(video),
              pictureInPicture: false,
          });
      });
      video.addEventListener('waiting', function () {
          trackMediaBufferStart({ id: id, player: updatePlayer(video) });
          isWaiting = true;
      });
      // `playing` is also triggered by playing the video after pausing,
      // so we need to check if the video was waiting before tracking the buffer
      video.addEventListener('playing', function () {
          if (isWaiting) {
              trackMediaBufferEnd({ id: id, player: updatePlayer(video) });
              isWaiting = false;
          }
      });
      video.addEventListener('error', function () {
          trackMediaError({ id: id, player: updatePlayer(video) });
      });
      video.addEventListener('timeupdate', function () {
          updateMediaTracking({ id: id, player: updatePlayer(video) });
      });
  }

  var LOG;
  function MediaTrackingPlugin() {
      return __assign(__assign({}, SnowplowMediaPlugin()), { logger: function (logger) {
              LOG = logger;
          } });
  }
  /**
   * Enables media tracking on an HTML `<video>` or `<audio>` element.
   *
   * @param {Config} config - Configuration options for media tracking.
   *
   * **Required:**
   * - `config.id` (`string`): The session ID. Must be unique for each media element. Used to identify the media session and end tracking.
   * - `config.video` (string | HTMLMediaElement): The ID of the media element or the element itself.
   *
   * **Optional:**
   * - `config.label` (`string`): A human-readable label for the media element.
   * - `config.captureEvents` {@link HTML5MediaEventTypes}: A list of media events to track. All events are tracked by default.
   * - `config.boundaries` (`number[]`): Percentage thresholds (0-100) at which to trigger progress events. Disabled by default.
   * - `config.context` {@link DynamicContext}: Contexts to attach to each tracking event.
   *
   * **Ping Configuration (Optional):**
   * - `config.pings.pingInterval` (`number`): The interval (in seconds) for sending ping events. Default is 30(s).
   * - `config.pings.maxPausedPings` (`number`): The maximum number of ping events sent while playback is paused. Default is 1.
   *
   * **Other Options (Optional):**
   * - `config.updatePageActivityWhilePlaying` (`boolean`): Whether to update page activity while media is playing. Enabled by default.
   * - `config.filterOutRepeatedEvents` {@link FilterOutRepeatedEvents}: Whether to suppress consecutive identical events. Default is false.
   */
  function startHtml5MediaTracking(config) {
      if (!config.video) {
          LOG.error("Missing 'video' property in the enableMediaTracking configuration. ", "Ensure the 'config.video' property is correctly set. Current config: ", config);
          return;
      }
      config = setConfigDefaults(config);
      if (isStringConfig(config)) {
          waitForElement(config, setUpListeners);
      }
      else if (isElementConfig(config)) {
          setUpListeners(config);
      }
  }
  /**
   * Ends media tracking with the given ID if previously started. Clears local state for the media tracking and sends any events waiting to be sent.
   *
   * @param {string} id - The session ID.
   */
  function endHtml5MediaTracking(id) {
      endMediaTracking({ id: id });
  }

  exports.MediaTrackingPlugin = MediaTrackingPlugin;
  exports.endHtml5MediaTracking = endHtml5MediaTracking;
  exports.startHtml5MediaTracking = startHtml5MediaTracking;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
